package co.f5s.pusher_connect;

import android.content.Context;
import android.os.Handler;
import android.os.Looper;

import java.util.HashMap;
import java.util.Map;

import com.pusher.client.Pusher;
import com.pusher.client.PusherOptions;
import com.pusher.client.channel.Channel;
import com.pusher.client.channel.SubscriptionEventListener;
import com.pusher.client.connection.ConnectionEventListener;
import com.pusher.client.connection.ConnectionState;
import com.pusher.client.connection.ConnectionStateChange;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/** PusherConnectPlugin */
public class PusherConnectPlugin implements MethodCallHandler, EventChannel.StreamHandler {
  private static final String METHOD_CHANNEL_NAME = "co.f5solutions/pusher_connect/channel";
  private static final String MESSAGE_RECIEVED_NAME = "co.f5solutions/pusher_connect/onRecieve";

  private Pusher pusher;
  private EventChannel.EventSink eventSink = null;

  private Handler handler;

  PusherConnectPlugin() {
    handler = new Handler(Looper.getMainLooper());
  }

  /** Plugin registration. */
  public static void registerWith(Registrar registrar) {
    PusherConnectPlugin pusherConnectPlugin = new PusherConnectPlugin();

    // Register methods
    final MethodChannel initChannel = new MethodChannel(registrar.messenger(), METHOD_CHANNEL_NAME);
    initChannel.setMethodCallHandler(pusherConnectPlugin);

    // Register the channels
    final EventChannel onRecieveChannel = new EventChannel(registrar.messenger(), MESSAGE_RECIEVED_NAME);
    onRecieveChannel.setStreamHandler(pusherConnectPlugin);
  }

  // onMethodCall for any methods needing to be queried
  @Override
  public void onMethodCall(MethodCall call, Result result) {
    if (call.method.equals("init")) {
      String appKey = call.argument("appKey");
      String cluster = call.argument("cluster");
      String channelToSubscribeTo = call.argument("channel");
      String eventToSubscribeTo = call.argument("event");

      PusherOptions options = new PusherOptions();
      options.setCluster(cluster);

      this.pusher = new Pusher(appKey, options); // TODO: Get from app
      System.out.println("Pusher Initialized");

      final Channel channel = this.pusher.subscribe(channelToSubscribeTo);

      channel.bind(eventToSubscribeTo, new SubscriptionEventListener() {
        @Override
        public void onEvent(final String channelName, final String eventName, final String data) {

          handler.post(new Runnable() {

            @Override
            public void run() {
              Map<String, Object> retMap = new HashMap();
              retMap.put("channelName", channelName);
              retMap.put("eventName", eventName);
              retMap.put("data", data);
              eventSink.success(retMap);
            }
          });
        }
      });

      this.pusher.connect();

      result.success(true);
    } else {
      result.notImplemented();
    }
  }

  // OnListen for any events to be called
  @Override
  public void onListen(Object arguments, final EventChannel.EventSink eventSink) {
    if (this.pusher == null) {
      eventSink.error("PUSHER_NOT_INITIALIZED", "Please initialize Pusher", null);
      return;
    }
    this.eventSink = eventSink;
  }

  @Override
  public void onCancel(Object arguments) {
    this.eventSink = null;
  }
}