import Flutter
import UIKit
import PusherSwift

public class SwiftPusherConnectPlugin: NSObject, FlutterPlugin, FlutterStreamHandler {
  var sinkOnChanged: FlutterEventSink?
  var listeningOnChanged = false
  var pusher: Pusher!

  public static func register(with registrar: FlutterPluginRegistrar) {
    let channel = FlutterMethodChannel(name: "co.f5solutions/pusher_connect/channel", binaryMessenger: registrar.messenger())
    let onRecieveChannel = FlutterEventChannel(name: "co.f5solutions/pusher_connect/onRecieve", binaryMessenger: registrar.messenger())

    let instance = SwiftPusherConnectPlugin()
    registrar.addMethodCallDelegate(instance, channel: channel)
    onRecieveChannel.setStreamHandler(instance)
  }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if call.method == "init" {
      guard let args = call.arguments else {
        result("No top level arguments")
        return
      }

      if let myArgs = args as? [String: Any],
      let appKey = myArgs["appKey"] as? String,
      let clusterToConnectTo = myArgs["cluster"] as? String,
      let channelToSubscribeTo = myArgs["channel"] as? String,
      let eventToSubscribeTo = myArgs["event"] as? String {

      let options = PusherClientOptions(
        host: .cluster(clusterToConnectTo)
      )

      pusher = Pusher(
        key: appKey,
        options: options
      )

      let channel = pusher.subscribe(channelToSubscribeTo)

      let _ = channel.bind(eventName: eventToSubscribeTo, callback: { (data: Any?) -> Void in

        
        // if data != nil {
        //   sinkOnChanged(data)
        // }
        if let data = data as? [String: AnyObject] {
            if self.sinkOnChanged != nil {
              var returnObject = [String : AnyObject]()
              returnObject["channelName"] = channelToSubscribeTo as AnyObject
              returnObject["eventName"] = eventToSubscribeTo as AnyObject
              returnObject["data"] = data as AnyObject
              self.sinkOnChanged!(returnObject)
            }
        }
      })

      pusher.connect()

      result(true)
      } else {
        result("Can not parse arguments")
      }
    } else {
      result("Nothing to do")
    }
  }

  public func onListen(withArguments arguments: Any?, eventSink events: @escaping FlutterEventSink) -> FlutterError? {
    self.sinkOnChanged = events
    return nil
  }

  public func onCancel(withArguments arguments: Any?) -> FlutterError? {
    self.sinkOnChanged = nil
    return nil
  }
}

class DummyDelegate: PusherDelegate {
  var pusher: Pusher!

    init() {
        let options = PusherClientOptions(
            authMethod: .inline(secret: "d3bb21ae30df1d3ee03c"),
            host: .cluster("eu")
        )
        pusher = Pusher(key: "c22921cba340ef079f75", options: options)
        pusher.connection.delegate = self // Online example shows this so I just added it in case
        pusher.delegate = self
        
        pusher.connect()
    }
    
    func changedConnectionState(from old: ConnectionState, to new: ConnectionState) {
        print("old: \(old.stringValue()) -> new: \(new.stringValue())")
    }
    
    private func debugLog(message: String) {
        print(message)
    }
}
