import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:flutter/services.dart';

class PusherConnect {
  MethodChannel _channel;
  EventChannel _onRecievedChannel;

  OnRecievedEvent _listenToOnRecieveEvent(dynamic map) {
    String channelName = map['channelName'];
    String eventName = map['eventName'];

    if (Platform.isIOS) {
      Map<dynamic, dynamic> data = map['data'];
      Map<String, dynamic> wantedData =
          data.map((a, b) => MapEntry(a as String, b as dynamic));

      return OnRecievedEvent(wantedData, channelName, eventName);
    }

    if (Platform.isAndroid) {
      String data = map['data'];
      Map<String, dynamic> wantedData = json.decode(data);

      return OnRecievedEvent(wantedData, channelName, eventName);
    }
    
    throw Exception("Platform not supported");
  }

  PusherConnect(
      {String appKey, String cluster = "eu", String channel, String event}) {
    _channel = new MethodChannel('co.f5solutions/pusher_connect/channel');
    _channel.invokeMethod('init', <String, dynamic>{
      'appKey': appKey,
      'cluster': cluster,
      'channel': channel,
      'event': event,
    }); // Init

    _onRecievedChannel =
        new EventChannel('co.f5solutions/pusher_connect/onRecieve');
  }

  Stream<OnRecievedEvent> get onRecievedEvents =>
      _onRecievedChannel.receiveBroadcastStream().map(_listenToOnRecieveEvent);
}

class OnRecievedEvent {
  final Map<String, dynamic> data;
  final String channel;
  final String event;

  OnRecievedEvent(this.data, this.channel, this.event);

  @override
  String toString() => 'OnRecievedEvent';
}
